#include <sdcard.h>
#include <sdraw.h>
#include <SPI.h>

SdRaw sdRaw;  // SD RAW class
SdSpi sdspi;  // SD SPI class

void setup()
{
  Serial.begin(9600);
  sdspi.pinSS((uChar)10);  // SD Select pin
  delay(2000);  // wait for Serial Monitor 8n1
  
  DRESULT res;  // FatFs return code
  uLong sector = firstSector;  // sector to read/write
  
  DSTATUS ds = sdRaw.initSd();  // init SD or test if memory card is inserted
  if (ds) showError("init", RES_NOTRDY);
  else
  {
    Serial.println(F("Fill the first data sector with char '-' ..."));
    Serial.println(F("   and show the first fifty bytes."));
    res = sdRaw.clearRaw(sector, '-');  
    if (res) showError("clear", res); 
    // read dashed sector back and show it 
    uChar buffRead[50];
    res = sdRaw.readRaw(buffRead, sector, 0, 50);
    if (res) showError("read buff", res);
    else showBlock(buffRead, 50);  // show the first 50 bytes
    
    // write array
    Serial.println(F("Write a char array ********** to offset 10 ..."));
    uChar buffWrite[] = {"**********"};
    res = sdRaw.writeRaw(buffWrite, sector, 10, sizeof(buffWrite));
    if (res) showError("write buff", res);  
    
    // write sting
    Serial.println(F("Write a String ########## to oftset 20 ..."));
    String str = "##########";
    res = sdRaw.writeString(str, sector, 20);  // write a block to SD Flash memory
    if (res) showError("write str", res); 

    // write number
    Serial.println(F("Write a unsigned long 1234567890 to offset 30 ..."));
    uLong n = 1234567890;
    res = sdRaw.writeString(String(n), sector, 30);  // write a block to SD Flash memory
    if (res) showError("write nbr", res); 

    Serial.println(F("   and show the first fifty bytes again ..."));
    res = sdRaw.readRaw(buffRead, sector, 0, 50);  // and read it back
    if (res) showError("read all back", res);
    else showBlock(buffRead, 50);
    Serial.println(F("__________________________________________________"));
    Serial.println(F("          1         2         3         4"));
    Serial.println(F("01234567890123456789012345678901234567890123456789"));
    Serial.println();
    Serial.println(F("Raw and files mixed? The data can also be found in the file."));
  }
}

void loop()
{
  //  delay(50);
}

void showBlock(uChar* buff, int count)
{
  for (int i = 0; i < count; i++) Serial.print((char)buff[i]);
  Serial.println();
}

void showError(String msg, DRESULT res)
{
  String result[] = {"Successful", "R/W Error", "Write Protected", "Not Ready", "Invalid Parameter"};
  Serial.print(msg); 
  Serial.print(", ERROR: "); 
  Serial.println(result[res]); 
}